; ============================================================================
; Měřič doby vysílání 8251 v asynchronním režimu
; - ve skutečnosti měří počet ticků, po kterém dojde k vyprázdnění 8251
; - počet ticků je následně přepočítán na milisekndy
; - s kódem je možno libovolně nakládat, šířit, modifikovat
; Autor: Zdeněk Šesták
; ============================================================================
uartdat           equ   01eh            ; datový registr obvodu 8251
uartcwr           equ   01fh            ; řídicí registr obvodu 8251
ctc_1ch           equ   05dh            ; data kanálu T1 obvodu 8253
ctc_cwr           equ   05fh            ; řídicí registr obvodu 8253
maska             equ   4               ; co sledujeme? TXEMPTY=4, TXRDY=1
osentry           equ   803eh           ; kam skočit po dokončení testu?

                  org   0
                  lxi   sp, 8000h
                  lxi   h, 0c200h
                  xra   a
                  sta   varPrtShift
                  shld  varCursor       ; nastavení pro microprint
                  mvi   a,1ch
                  call  8500h           ; maže obrazovku
                  lxi   h,titletxt
                  call  printtext       ; vypíše titulek

                  ; nastavení čítače T1 u 8253 (na frekvenci vlastně ani nezáleží,
                  ; když počítáme ticky, proto je zvolena vyšší pro rychlejší
                  ; měření na reálném PMD))
                  mvi   a,76h
                  out   ctc_cwr
                  mvi   a,080h
                  out   ctc_1ch
                  mvi   a,01h
                  out   ctc_1ch
                  ; smyčka volby režimu
                  mvi   c,00h           ; c = číslo režimu
newline:          call  printcr
                  mov   a,c
                  rrc
                  rrc
                  rrc
                  rrc
                  call  hexToChar
pnum:             call  print
                  mvi   a,'x'
                  call  print
                  mvi   a,'|'
                  call  print
                  inr   c
measure:          push  b
                  pop   b
                  push  b
                  call  start           ; test režimu
                  pop   b
skipmode:         inr   c
                  jz    osentry         ; všechny režimy přeskenovány
                  mov   a,c
                  ani   0fh
                  jz    newline
                  mov   a,c
                  ani   3
                  jz    skipmode        ; přeskakuje synchronní mód
                  jmp   measure

hexToChar:        ani   00001111B
                  cpi   10
                  sbi   2FH
                  daa
                  ret
; měřící smyčka
; c = mód
; výsledkem je vypsaný text
start:            ; setup 8251
                  mvi   a,82h           ; abychom měli jistotu, že 40h bude reset
                  out   uartcwr
                  out   uartcwr
                  mvi   a,40h           ; RESET
                  out   uartcwr
                  mov   a,c
                  out   uartcwr         ; nastavení režimu
                  mvi   a,021h
                  out   uartcwr         ; povolit vysílání znaku

                  ; vyšleme bajt a až druhý/třetí bajt změříme
                  out   uartdat
synchro:          in    uartcwr
                  ani   maska
                  jz    synchro
                  if    maska == 1
                  out   uartdat         ; při synchronizaci TXRDY
synchro2:         in    uartcwr         ; je vhodné přeplnit vstupní buffery
                  ani   maska           ; ještě jedním vysíláním
                  jz    synchro2
                  endif
                  lxi   h, 0            ; počítadlo taktů 8251
                  out   uartdat         ; vyšle data
wait8251:         call  timOVF          ; počká na přetečení čítače
                  call  timOVF          ; počká na přetečení čítače
                  inx   h
                  in    uartcwr         ; načte stavové slovo
                  ani   maska           ; TxEMPTY/TXRDY?
                  jz    wait8251        ; UARTu není vyprázdněn, tak ne..
                  ; v tuto chvíli je v HL počet taktů vysílání slova
                  ; doba = HL * 8333 / 10000
                  ; příprava textu - vyplní znakem nula
                  xchg
                  lxi   h, timetxt
                  mvi   a, 7
clear:            mvi   m,'0'
                  inx   h
                  dcr   a
                  jnz   clear
                  ; přičítání čísla 8333 = 1/1200 * 10 000 000
                  ; = doba trvání jednoto taktu v desetinách mikrosekund
toNumber:         lxi   h, timetxt+6
                  mvi   a, 3
                  call  addnum
                  mvi   a, 3
                  call  addnum
                  mvi   a, 3
                  call  addnum
                  mvi   a, 8
                  call  addnum
                  call  repair
                  dcx   d
                  mov   a,e
                  ora   d
                  jnz   toNumber
                  ; zaokrouhlení na 1 desetinné místo
                  lxi   h,timetxt+4
                  call  round
noround:          ; potlačení nul na začátku
                  lxi   h, timetxt-1
                  db    3eh             ; mvi a,
zero:             mov   m,a
                  inx   h
                  mov   a,m
                  cpi   '0'
                  mvi   a,' '
                  jz    zero
                  lda   timetxt+0
                  cpi   ' '             ; trojmístné číslo?
                  jnz   big
                  ; malá čísla - úprava
                  ; ..123
                  ; ..12,3
                  lxi   h, timetxt + 3
                  mov   a,m
                  mvi   m, ','
                  inx   h
                  mov   m,a
                  inx   h
                  mvi   m,0
                  lxi   h,timetxt
                  jmp   printtext
big:              lxi   h,timetxt+3                  
                  call  round           ; velká čísla se zaokrouhlí na jednotky
                  xra   a
                  sta   timetxt + 3
                  lxi   h, timeprt
                  jmp   printtext
; čeká na přetečení čítače
; takt 8251 = 2x přetečení (v módu 3 se dekrementuje po dvou)
timOVF:           in    ctc_1ch         ; low
                  in    ctc_1ch         ; high
                  ana   a
                  jnz   timOVF
.w:               in    ctc_1ch
                  in    ctc_1ch
                  ana   a
                  jz    .w
                  ret
; přičte číslo v A k číslici (znaku) na HL a zpracuje přenosy
addnum:           add   m
                  mov   m,a
                  cpi   '9'+1
                  dcx   h
                  rc
                  sui   10
                  inx   h
                  mov   m,a
                  dcx   h
                  inr   m
                  ret
; zaokrouhlí číslo od místa, kam ukazuje hl
round:            mov   a,m
                  cpi   '5'
                  rc
                  dcx   h
                  inr   m
                  jmp   repair
; opraví číslo, pokud některá z cifer přesáhla 9
repair:           lxi   h, timetxt+5
rloop:            mov   a,m
                  cpi   '0'
                  jnc   checkspace
                  cpi   ' '
                  jz    checkspace
                  adi   -32+48          ; např. přenos o jedničku na mezeru udělá vykřičník
                  mov   m, a
checkspace:       xra   a
                  call  addnum
                  mov   a,l
                  cpi   (timetxt-1) & 255
                  jnz   rloop
                  ret
; tisk
printcr:          mvi   a,13
                  jmp   print
printspace:       mvi   a,32
print:            jmp   microPrint      ;8500h
printtext:        mov   a,m
                  inx   h
                  ana   a
                  rz
                  call  print
                  jmp   printtext

timeprt:          db    "  "
timetxt:          db    "0000000",0
titletxt:         db    "Doba vyprazdneni vysilaciho registru UARTu pro jednotlive rezimy [msec]",13,13
                  db    "     x1   x2   x3   x5   x6   x7   x9   xA   xB   xD   xE   xF",13
                  db    "   ------------------------------------------------------------",0
;---------------------------------------------------------------------------
; microPrint - zobrazí jeden malý znak metodou XOR (rastr 4x8 pixelů)
;---------------------------------------------------------------------------
varCursor:        dw    0c200h          ; kurzor
varPrtLeft:       db    0, 47           ; okraje levý a pravý
varPrtShift       db    0               ; posun v rámci bajtu
varSP:            ds    2               ; úložiště SP

microprint:       ana   a
                  rz
                  push  h
                  push  d
                  push  b
                  ; nový řádek?
                  cpi   13
                  jnz   .mp
                  lhld  varCursor
                  inr   h
                  inr   h
                  mov   a,l
                  ani   0c0h
                  mov   l,a
                  lda   varPrtLeft
                  ora   l
                  mov   l,a
                  shld  varCursor
                  xra   a
                  sta   varPrtShift
                  jmp   .prtex

                  ; vypočítá adresu fontu do de
.mp:              sui   32
                  mov   c,a
                  ani   0feh
                  mov   l,a
                  mvi   h,0
                  dad   h
                  dad   h
                  lxi   d, gfxFont
                  dad   d
                  push  h

                  ; kontrola, zda je možné do daného místa vykreslit
                  lhld  varPrtLeft
                  xchg
                  lhld  varCursor
                  mov   a,l
                  ani   3fh
                  cmp   d
                  jc    .ppp
                  xra   l
                  ora   e
                  mov   l,a
                  inr   h
                  inr   h
                  shld  varCursor
                  xra   a
                  sta   varPrtShift
.ppp:             pop   d
                  di
                  lxi   h,0
                  dad   sp
                  shld  varSP
                  lxi   h, varPrtShift
                  lxi   sp, 64

                  mov   a,c
                  mvi   c,8
                  rrc
                  jc    .podd
                  ; b - maska pro čtení dat fontu
                  mvi   b, 0fh
                  dcr   m
                  jm    .prt0
                  lhld  varCursor
                  jz    .prt1
                  dcx   sp
                  mvi   b,30h
.p2:              ldax  d
                  rrc
                  rrc
                  rrc
                  rrc
                  ana   b
                  xra   m
                  mov   m,a
                  inx   h
                  ldax  d
                  rrc
                  rrc
                  ani   3
                  xra   m
                  mov   m,a
                  inx   d
                  dad   sp
                  dcr   c
                  jnz   .p2
                  jmp   .prtexitshft
.prt1:
                  ldax  d
                  inx   d
                  ana   b
                  rlc
                  rlc
                  xra   m
                  mov   m,a
                  dad   sp
                  dcr   c
                  jnz   .prt1
.prtexitshft:     lxi   h,varCursor
                  inr   m
.prtexit:         lhld  varsp
                  sphl
                  ei
.prtex:           pop   b
                  pop   d
                  pop   h
                  ret

.prt0:            mvi   m,2
                  lhld  varCursor

.p0:              ldax  d
                  inx   d
                  ana   b
                  xra   m
                  mov   m,a
                  dad   sp
                  dcr   c
                  jnz   .p0

                  jmp   .prtexit

                  ; b - maska pro čtení dat fontu
.podd:            mvi   b, 0f0h
                  dcr   m
                  jm    .pprt0
                  lhld  varCursor
                  jz    .pprt1
                  dcx   sp
                  lxi   b,30h*256+8
.pp2:             ldax  d
                  ana   b
                  xra   m
                  mov   m,a
                  inx   h
                  ldax  d
                  rlc
                  rlc
                  ani   3
                  xra   m
                  mov   m,a
                  inx   d
                  dad   sp
                  dcr   c
                  jnz   .pp2
                  jmp   .prtexitshft

.pprt1:
                  ldax  d
                  inx   d
                  ana   b
                  rrc
                  rrc
                  xra   m
                  mov   m,a
                  dad   sp
                  dcr   c
                  jnz   .pprt1

                  jmp   .prtexitshft

.pprt0:           mvi   m,2
                  lhld  varCursor
.pp0:             ldax  d
                  inx   d
                  ana   b
                  rrc
                  rrc
                  rrc
                  rrc
                  xra   m
                  mov   m,a
                  dad   sp
                  dcr   c
                  jnz   .pp0
                  jmp   .prtexit



gfxFont:
                  DB    0000000B
                  DB    0100000B
                  DB    0100000B
                  DB    0100000B
                  DB    0100000B
                  DB    0000000B
                  DB    0100000B
                  DB    0000000B


                  DB    0000000B
                  DB    1010101B
                  DB    1110101B
                  DB    1010000B
                  DB    1010000B
                  DB    1110000B
                  DB    1010000B
                  DB    0000000B

                  DB    0000010B
                  DB    1010111B
                  DB    1000001B
                  DB    0100111B
                  DB    0100100B
                  DB    0010111B
                  DB    1010010B
                  DB    0000000B

                  DB    0000000B
                  DB    1100010B
                  DB    1000101B
                  DB    0100010B
                  DB    0001101B
                  DB    0000101B
                  DB    0001010B
                  DB    0000000B

                  DB    0000000B
                  DB    0010100B
                  DB    0100010B
                  DB    1000001B
                  DB    1000001B
                  DB    0100010B
                  DB    0010100B
                  DB    0000000B

                  DB    0000000B
                  DB    0000000B
                  DB    0100101B
                  DB    0100010B
                  DB    1110111B
                  DB    0100010B
                  DB    0100101B
                  DB    0000000B

                  DB    0000000B
                  DB    0000000B
                  DB    0000000B
                  DB    0000000B
                  DB    1110000B
                  DB    0000110B
                  DB    0000100B
                  DB    0000010B

                  DB    0000000B
                  DB    1000000B
                  DB    1000000B
                  DB    0100000B
                  DB    0100000B
                  DB    0010110B
                  DB    0010110B
                  DB    0000000B

                  DB    0000000B
                  DB    0100010B
                  DB    0110101B
                  DB    0100101B
                  DB    0100101B
                  DB    0100101B
                  DB    1110010B
                  DB    0000000B

                  DB    0000000B
                  DB    1110010B
                  DB    1000101B
                  DB    0100100B
                  DB    1000010B
                  DB    1010001B
                  DB    0100111B
                  DB    0000000B

                  DB    0000000B
                  DB    1110100B
                  DB    0010110B
                  DB    0110101B
                  DB    1000101B
                  DB    1010111B
                  DB    0100100B
                  DB    0000000B

                  DB    0000000B
                  DB    1110110B
                  DB    1000001B
                  DB    1000011B
                  DB    0100101B
                  DB    0100101B
                  DB    0100010B
                  DB    0000000B

                  DB    0000000B
                  DB    0100010B
                  DB    1010101B
                  DB    1010010B
                  DB    1100101B
                  DB    1000101B
                  DB    0110010B
                  DB    0000000B

                  DB    0000000B
                  DB    0000000B
                  DB    1100110B
                  DB    1100110B
                  DB    0000000B
                  DB    1100110B
                  DB    1000110B
                  DB    0100000B

                  DB    0000000B
                  DB    0000000B
                  DB    0000100B
                  DB    1110010B
                  DB    0000001B
                  DB    1110010B
                  DB    0000100B
                  DB    0000000B

                  DB    0000000B
                  DB    0100000B
                  DB    1010001B
                  DB    1000010B
                  DB    0100100B
                  DB    0000010B
                  DB    0100001B
                  DB    0000000B


                  DB    0000000B
                  DB    0100110B
                  DB    1011001B
                  DB    1011101B
                  DB    1111101B
                  DB    1010001B
                  DB    1010110B
                  DB    0000000B

                  DB    0000000B
                  DB    1100011B
                  DB    0010101B
                  DB    0010011B
                  DB    0010101B
                  DB    0010101B
                  DB    1100011B
                  DB    0000000B

                  DB    0000000B
                  DB    1110011B
                  DB    0010101B
                  DB    0110101B
                  DB    0010101B
                  DB    0010101B
                  DB    1110011B
                  DB    0000000B

                  DB    0000000B
                  DB    1100111B
                  DB    0010001B
                  DB    0010011B
                  DB    1110001B
                  DB    1010001B
                  DB    1100001B
                  DB    0000000B

                  DB    0000000B
                  DB    1110101B
                  DB    0100101B
                  DB    0100111B
                  DB    0100101B
                  DB    0100101B
                  DB    1110101B
                  DB    0000000B

                  DB    0000000B
                  DB    1010100B
                  DB    1010100B
                  DB    0110100B
                  DB    1010101B
                  DB    1010101B
                  DB    1010010B
                  DB    0000000B

                  DB    0000000B
                  DB    1010001B
                  DB    1110001B
                  DB    1110001B
                  DB    1010001B
                  DB    1010001B
                  DB    1010111B
                  DB    0000000B

                  DB    0000000B
                  DB    1110011B
                  DB    1010101B
                  DB    1010101B
                  DB    1010101B
                  DB    1010101B
                  DB    1110101B
                  DB    0000000B

                  DB    0000000B
                  DB    1110111B
                  DB    1010101B
                  DB    1010101B
                  DB    1010111B
                  DB    1010001B
                  DB    0110001B
                  DB    1000000B

                  DB    0000000B
                  DB    1110111B
                  DB    0010101B
                  DB    1110101B
                  DB    1000111B
                  DB    1000011B
                  DB    1110101B
                  DB    0000000B

                  DB    0000000B
                  DB    1010111B
                  DB    1010010B
                  DB    1010010B
                  DB    1010010B
                  DB    1010010B
                  DB    1110010B
                  DB    0000000B

                  DB    0000000B
                  DB    1010101B
                  DB    1010101B
                  DB    1010101B
                  DB    1110101B
                  DB    1110101B
                  DB    1010010B
                  DB    0000000B

                  DB    0000000B
                  DB    1010101B
                  DB    1010101B
                  DB    1010010B
                  DB    0100010B
                  DB    0100101B
                  DB    0100101B
                  DB    0000000B

                  DB    0000000B
                  DB    1110111B
                  DB    0010100B
                  DB    0010010B
                  DB    0010010B
                  DB    0010001B
                  DB    1110111B
                  DB    0000000B

                  DB    0000000B
                  DB    1110001B
                  DB    1000001B
                  DB    1000010B
                  DB    1000010B
                  DB    1000100B
                  DB    1110100B
                  DB    0000000B

                  DB    0000000B
                  DB    0000010B
                  DB    0000101B
                  DB    0000000B
                  DB    0000000B
                  DB    0000000B
                  DB    11110000B
                  DB    0000000B

                  DB    0000010B
                  DB    0000100B
                  DB    1110000B
                  DB    1000000B
                  DB    1110000B
                  DB    1010000B
                  DB    1110000B
                  DB    0000000B

                  DB    0000000B
                  DB    0000001B
                  DB    1100001B
                  DB    0010111B
                  DB    0010101B
                  DB    0010101B
                  DB    1100111B
                  DB    0000000B

                  DB    0000000B
                  DB    0000100B
                  DB    0100100B
                  DB    1010111B
                  DB    1110101B
                  DB    0010101B
                  DB    1100111B
                  DB    0000000B

                  DB    0000000B
                  DB    0000110B
                  DB    0000010B
                  DB    1110111B
                  DB    1010010B
                  DB    1110010B
                  DB    1000010B
                  DB    1110000B

                  DB    0000000B
                  DB    0100001B
                  DB    0000001B
                  DB    0110111B
                  DB    0100101B
                  DB    0100101B
                  DB    1110101B
                  DB    0000000B

                  DB    0000000B
                  DB    0010100B
                  DB    1010000B
                  DB    1010110B
                  DB    0110100B
                  DB    1010100B
                  DB    1010101B
                  DB    0000010B

                  DB    0000000B
                  DB    0000011B
                  DB    1010010B
                  DB    1110010B
                  DB    1110010B
                  DB    1010010B
                  DB    1010111B
                  DB    0000000B

                  DB    0000000B
                  DB    0000000B
                  DB    1110011B
                  DB    1010101B
                  DB    1010101B
                  DB    1010101B
                  DB    1110101B
                  DB    0000000B

                  DB    0000000B
                  DB    0000000B
                  DB    1110111B
                  DB    1010101B
                  DB    1010101B
                  DB    1110111B
                  DB    1000001B
                  DB    1000001B

                  DB    0000000B
                  DB    0000000B
                  DB    1110111B
                  DB    0010001B
                  DB    1110001B
                  DB    1000001B
                  DB    1110001B
                  DB    0000000B

                  DB    0000000B
                  DB    0000010B
                  DB    1010111B
                  DB    1010010B
                  DB    1010010B
                  DB    1010010B
                  DB    1110110B
                  DB    0000000B

                  DB    0000000B
                  DB    0000000B
                  DB    1010101B
                  DB    1010101B
                  DB    1110101B
                  DB    1110101B
                  DB    1010010B
                  DB    0000000B

                  DB    0000000B
                  DB    0000000B
                  DB    1010101B
                  DB    1010101B
                  DB    1010010B
                  DB    1110101B
                  DB    1000101B
                  DB    1110000B

                  DB    1000000B
                  DB    0100000B
                  DB    0100111B
                  DB    0010100B
                  DB    0100010B
                  DB    0100001B
                  DB    1000111B
                  DB    0000000B

                  DB    0010000B
                  DB    0100010B
                  DB    0100010B
                  DB    1000010B
                  DB    0100010B
                  DB    0100010B
                  DB    0010010B
                  DB    0000000B

                  DB    11110000B
                  DB    11110000B
                  DB    11110000B
                  DB    11111010B
                  DB    11110101B
                  DB    11110000B
                  DB    11110000B
                  DB    11110000B

                  end